-------------------------------------------------------------------------------
-- Dark Star One (TM) - Script library
-- Copyright (C) Ascaron Entertainment GmbH
-- All Rights Reserved.
------------------------------[ Info ]-----------------------------------------
-- Author: Peter Grimsehl
------------------------------[ Description ]----------------------------------
-- Utility functions for instantiating large battles
------------------------------[ Import ]---------------------------------------
source "lua/mission/MissionLib.lua"
source "lua/mission/CameraLib.lua"
-------------------------------------------------------------------------------

BattleLib = {}

-- Erzeugt ein Schlachtfeld
BattleLib.CreateBG = function( Data )
	local bg  = {}
	bg.Wings  = {}
	bg.Defs   = Data.Defs
	bg.Center = Data.Center
	bg.Groups = NGroupAi.Create({})
	return bg
end


-- Aktualisiert die Gegnerlisten aller erzeugten Wings und trgt aktuelle Handles ein
BattleLib.UpdateEnemies = function( BG )
	for name, def in BG.Defs do
		if BG.Wings[name] then
			local enemies = def.Enemies
			local loop = 1
			while enemies and enemies[loop] do
				if BG.Wings[enemies[loop].Target] then
					NWing.AddEnemy( {
						Wing	 = BG.Wings[name],
						Target	 = BG.Wings[enemies[loop].Target],
						Priority = enemies[loop].Priority
					} )
				elseif "ActionTurret" == enemies[loop].Target then
					NWing.AddEnemy( {
						Wing	 = BG.Wings[name],
						Target	 = NObject.GetActionTurret( {} ).Handle,
						Priority = enemies[loop].Priority
					} )
				elseif "Player" == enemies[loop].Target then
					NWing.AddEnemy( {
						Wing	 = BG.Wings[name],
						Target	 = MissionLib.GetPlayerHandle(),
						Priority = enemies[loop].Priority
					} )
				end
				loop = loop + 1
			end
		end
	end
end


-- Entfernt einen Wing aus der WingListe
BattleLib.UpdateWingList = function( BG, Wing )
--	NDebug.Message( { Message = "Updating wing list" } )
	local id = MissionLib.IsInList( Wing, BG.Wings )
	if id then
		BG.Wings[id] = nil
--		NDebug.Message( { Message = "Removed wing "..id } )
	end
end


-- Erzeugt einen Wing anhand einer Wingdefinition
BattleLib.CreateWing = function( BG, Data )
--	NDebug.Message( { Message = "Creating wing "..Data.WingId } )

	local wing_id = Data.WingId
	local restore = Data.Restore

	if not BG.Defs[wing_id] or BG.Wings[wing_id] then
		return
	end

	local wing_def = BG.Defs[wing_id]
	local use_define
	local def = wing_def.Create

	if not def then
		def = wing_def.Define
		use_define = true
	end

	if restore then
		BG.Wings[wing_id] = NWing.Create( {
			WingType = def.WingType,
			Number	 = restore.Number,
			PosType	 = POSDESCR_EXPLICIT,
			Position = restore.Position,
			Race	 = def.Race,
			Names	 = def.Names
		} ).Wing
	else
		local temp = MissionLib.CopyTable( def )
		if def.Object then
			if BG.Wings[def.Object] then
				temp.Object = NWing.List( {
					Wing = BG.Wings[def.Object]
				} ).ShipList[1]
			elseif "TradeStation" == def.Object then
				temp.Object = MissionLib.GetSystemTradeStation()
			elseif "ResearchStation" == def.Object then
				temp.Object = MissionLib.GetSystemResearchStation()
			else
				-- Wing kann hier nicht erzeugt werden
--				NDebug.Message( { Message = "Wing "..wing_id.." konnte nicht erzeugt werden. Objekt mit Hangar nicht vorhanden" } )
				return
			end
		end
		if def.Position then
			temp.Position = NVector.Add( {
				Vector1 = BG.Center,
				Vector2 = def.Position
			} ).Vector
		end

		if use_define then
			BG.Wings[wing_id] = NWing.Define( temp ).Wing
		else
			BG.Wings[wing_id] = NWing.Create( temp ).Wing
		end
	end

	-- Zustzliche Feindtypen eintragen
	if wing_def.EnemyTypes then
		for id, type_id in wing_def.EnemyTypes do
			NWing.SetInvalidEnemy( {
				Wing = BG.Wings[wing_id],
				TypeId = type_id,
				Set = 0
			} )
		end
	end

	-- Feindlisten aller erzeugten Wings aktualisieren
	BattleLib.UpdateEnemies( BG )

	-- Einmal die Liste aller Schiffe des Wings ermitteln
	local ship_list = NWing.List( { Wing = BG.Wings[wing_id] } ).ShipList

	-- Spieler soll sofort feindlich sein
	if "enemy" == Data.Player then
		NWing.AddEnemy( {
			Wing = BG.Wings[wing_id],
			Target = MissionLib.GetPlayerHandle()
		} )
	-- Spielerbeschuss wird weniger ernst genommen
	elseif "friend" == Data.Player then
		for index, ship in ship_list do
			NShip.SetPlayerFriend( { Ship = ship, Set = true } )
		end
	-- Spieler wird niemals feindlich
	elseif "invalid" == Data.Player then
		NWing.SetInvalidEnemy( { Wing = BG.Wings[wing_id], TypeId = ST_PLAYERSHIP } )
	end

	-- Gruppenzugehrigkeit
	local group = wing_def.Group
	if group and BG.Groups then
		NGroupAi.AddWings( {
			Group = BG.Groups[group],
			WingList = { BG.Wings[wing_id] }
		} )
	end

	-- Schiffe markieren
	if "all" == wing_def.Marker then
		MissionLib.SetMissionFlagOnWing( BG.Wings[wing_id], true )
	elseif wing_def.Marker then
		for loop = 1, wing_def.Marker, 1 do
			if ship_list[loop] then
				NObject.SetMissionFlag( { Object = ship_list[loop], Set = true } )
			end
		end
	end

	-- Strafpunkte bei Beschuss
	if wing_def.Crime then
		for index, ship in ship_list do
			NShip.SetCrime( { Ship = ship, Set = true } )
		end
	end

	-- Boss?
	if wing_def.BossId then
		local loop = 1
		while ship_list[loop] do
			if wing_def.BossId[loop] then
				NShip.SetBoss( {
					Ship = ship_list[loop],
					BossId = wing_def.BossId[loop]
				} )
			end
			loop = loop + 1
		end
	end

	-- Spezielle Ausrichtung
	local align = wing_def.Alignment
	-- nachtrgliche bergabe hat Prioritt
	if Data.Alignment then
		align = Data.Alignment
	end
	if align then
		-- Lookat in BG Koordinaten?
		if ( true == align.Relative )
		and ( def.Position ) then
			align.Front = NVector.Sub( {
				Vector1 = align.Front,
				Vector2 = def.Position
			} ).Vector
		end
		for index, ship in ship_list do
			NShip.SetAlignment( {
				Ship = ship,
				Front = align.Front,
				UpDir = align.UpDir
			} )
		end
	end

	-- Akteure
	if wing_def.Actor then
		for index, ship in ship_list do
			NShip.SetActor( { Ship = ship, Actor = wing_def.Actor } )
		end
	elseif wing_def.ActorList then
		local loop = 1
		while ship_list[loop] do
			if wing_def.ActorList[loop] then
				NShip.SetActor( {
					Ship = ship_list[loop],
					Actor = wing_def.ActorList[loop]
				} )
			end
			loop = loop + 1
		end
	end

	-- Aktionen fr den neuen Wing kopieren
	local loop = 1
	if wing_def.Actions then
		while wing_def.Actions[loop] do
			local action = wing_def.Actions[loop]
			-- Handlewerte konkretisieren
			local action_list = {}
			local index = 1
			local inner_loop = 1
			while action.Actions[inner_loop] do
				local add = MissionLib.CopyTable( action.Actions[inner_loop] )
				-- Hier muss ein Bezeichner in ein Handlewert verwandelt werden
				if add.Handle then
					-- Spezialfall landen
					if AI_ACTION_LANDEN == add.Type then
						-- Handelstation
						if "TradeStation" == add.Handle then
							add.Handle = MissionLib.GetSystemTradeStation()
						elseif "Player" == add.Handle then
							add.Handle = MissionLib.GetPlayerHandle()
						-- Ein Schiff mit Hangar
						elseif BG.Wings[add.Handle] then
							add.Handle = NWing.List( {
								Wing = BG.Wings[add.Handle] } ).ShipList[1]
						-- geht nicht
						else
							add = nil
						end
					elseif BG.Wings[add.Handle] then
						add.Handle = BG.Wings[add.Handle]
					else
						add = nil
					end
				-- Position in BG Koordinaten umrechnen
				elseif add.Position then
					add.Position = NVector.Add( {
						Vector1 = BG.Center,
						Vector2 = add.Position
					} ).Vector
				end
				-- Die Einzelaktion bernehmen
				if add then
					action_list[index] = add
					index = index + 1
				end
				inner_loop = inner_loop + 1
			end
			-- Die Aktionsliste bernehmen
			NWing.AddAction( {
				Wing = BG.Wings[wing_id],
				First = action.First,
				List = action.List,
				Actions = action_list
			} )
			loop = loop + 1
		end
	end

	-- Beladen falls gewnscht
	if wing_def.Cargo then
		if "Random" == wing_def.Cargo then
			for id, ship in ship_list do
				NShip.CreateRandomCargo( { Ship = ship } )
			end
		else
			for id, ship in ship_list do
				local bays = NShip.GetNumCargoBays( { Ship = ship } ).NumCargoBays
				for i = 1, bays, 1 do
					NShip.EquipContainer( { Ship = ship, Goods = wing_def.Cargo } )
				end
			end
		end
	end

	-- Ladung sichern
	if wing_def.ProtectCargo then
		for id, ship in ship_list do
			NShip.ProtectCargo( { Ship = ship, Protect = true } )
		end
	end

--	NDebug.Message( { Message = "Wing "..Data.WingId.." successfully created" } )
end


BattleLib.SendMessage = function( BG, pSender, pText, pMsgOnly )
	local in_out = true
	if pMsgOnly then
		in_out = nil
	end

	if BG.Wings[pSender] then
		if NWing.IsWing( { Wing = BG.Wings[pSender] } ).IsWing == 1 then
			return NComm.AddMessage( {
				Sender = BG.Wings[pSender],
				Text = pText,
				CommInOut = in_out
			} ).Message
		else
			return nil
		end
	elseif "TradeStation" == pSender then
		return NComm.AddMessage( {
			Sender = MissionLib.GetSystemTradeStation(),
			Text = pText,
			CommInOut = in_out
		} ).Message
	elseif "ResearchStation" == pSender then
		return NComm.AddMessage( {
			Sender = MissionLib.GetSystemResearchStation(),
			Text = pText,
			CommInOut = in_out
		} ).Message
	elseif "Player" == pSender then
		msg = NComm.AddMessage( {
			Sender = MissionLib.GetPlayerHandle(),
			Text = pText,
			CommInOut = in_out
		} ).Message
	end

	return nil
end


BattleLib.CopilotMessage = function( pText )
	if NPlayer.IsEona({}).Eona then
		return NComm.AddMessage( {
			Sender = MissionLib.GetPlayerHandle(),
			Text = pText,
			Actor = "eona",
			CommInOut = false
		} ).Message
	end
end


BattleLib.KayronMessage = function( pText )
	return NComm.AddMessage( {
		Sender = MissionLib.GetPlayerHandle(),
		Text = pText,
		Actor = "kayron",
		CommInOut = false
	} ).Message
end


BattleLib.SendCamMessage = function( BG, pSender, pTarget, pText, pCamType, pZoom )
	local msg, cam

	if BG.Wings[pSender] then
		msg = NComm.AddMessage( {
			Sender = BG.Wings[pSender],
			Text = pText,
			CommInOut = false
		} ).Message
	elseif "TradeStation" == pSender then
		msg = NComm.AddMessage( {
			Sender = MissionLib.GetSystemTradeStation(),
			Text = pText,
			CommInOut = false
		} ).Message
	elseif "Player" == pSender then
		msg = NComm.AddMessage( {
			Sender = MissionLib.GetPlayerHandle(),
			Text = pText,
			CommInOut = false
		} ).Message
	end

	if BG.Wings[pTarget] then
		if pCamType and CameraLib[pCamType] then
			cam = CameraLib[pCamType]( BG.Wings[pTarget], 2000.0, pZoom )
		else
			cam = CameraLib.TalkCamLeader( BG.Wings[pTarget], 2000.0 )
		end
	elseif "TradeStation" == pSender then
		cam = CameraLib.ShowStation( {0,0,0}, 2000.0 )
	end

	return cam, msg
end


BattleLib.IsGroupDestroyed = function( BG, pGroup )
	for index, def in BG.Defs do
		if BG.Wings[index] then
			if BG.Defs[index].Group == pGroup then
				return false
			end
		end
	end
	return true
end


BattleLib.FreezeGroup = function( BG, pGroup, pFreeze )
	for index, def in BG.Defs do
		if BG.Wings[index] then
			if BG.Defs[index].Group == pGroup then
				MissionLib.FreezeWing( BG.Wings[index], pFreeze )
			end
		end
	end
end


BattleLib.ProtectWing = function( BG, pWing, pFromPlayer )
	if pWing and BG.Wings[pWing] then
		local ship_list = NWing.List( { Wing = BG.Wings[pWing] } ).ShipList
		for id, ship in ship_list do
			if pFromPlayer then
				NWing.SetInvalidEnemy( {
					Wing = BG.Wings[pWing],
					TypeId = ST_PLAYERSHIP
				} )
			end
			NShip.Protect( { Ship = ship, Protect = true } )
		end
	end
end


BattleLib.ProtectGroup = function( BG, pGroup )
	for index, def in BG.Defs do
		if def.Group == pGroup then
			BattleLib.ProtectWing( BG, index )
		end
	end
end


BattleLib.RemoveWing = function( BG, pWing )
	if pWing and BG.Wings[pWing] then
		if BG.Groups and BG.Defs[pWing] and BG.Defs[pWing].Group then
			NGroupAi.SubWings( {
				Group = BG.Groups[BG.Defs[pWing].Group],
				WingList = { BG.Wings[pWing] }
			} )
		end
		--BG.Wings[pWing] = nil
	end
end


BattleLib.DeleteGroups = function( BG )
	if BG and BG.Groups then
		NGroupAi.Destroy( { Group = BG.Groups.GroupA } )
		NGroupAi.Destroy( { Group = BG.Groups.GroupB } )
		BG.Groups = nil
	end
end


BattleLib.GetWingId = function( BG, pWingHandle )
	for id, wing in BG.Wings do
		if wing == pWingHandle then
			return id
		end
	end
	return "???"
end


BattleLib.GroupAction = function( BG, pGroup, pAction, pHandle )
	for id, wing in BG.Wings do
		if BG.Defs[id].Group == pGroup then
			NWing.AddAction( {
				Wing = wing,
				List = LIST_ACTIVE,
				First = 1,
				Actions = { { Type = pAction, Handle = pHandle } }
			} )
			NWing.ExecuteAi( { Wing = wing } )
		end
	end
end


BattleLib.DestroyGroup = function( BG, pGroup )
	local id_table = {}
	for id, wing in BG.Wings do
		if BG.Defs[id].Group == pGroup then
			id_table[id] = wing
			if NWing.IsWing( { Wing = wing } ).IsWing == 1 then
				NWing.Destroy( { Wing = wing } )
			end
		end
	end
	for id, wing in id_table do
		BG.Wings[id] = nil
	end
end


BattleLib.IsWingInGroup = function( BG, pWing, pGroup )
	if not BG then
		return false
	end
	for id, wing in BG.Wings do
		if wing == pWing then
			if BG.Defs[id].Group == pGroup then
				return true
			end
		end
	end
	return false
end


BattleLib.HyperOut = function( BG, pWing, pImmediate )
	if BG.Wings[pWing] then
		if NWing.IsWing( { Wing = BG.Wings[pWing] } ).IsWing == 1 then
			local action = AI_ACTION_HYPERRAUM
			if pImmediate then
				action = AI_ACTION_SOFORTSPRUNG
			end
			NWing.AddAction( {
				Wing = BG.Wings[pWing],
				List = LIST_ACTIVE,
				First = true,
				Actions = { { Type = action } }
			} )
			NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
		end
	end
end


BattleLib.Goto = function( BG, pWing, pPosition )
	if BG.Wings[pWing] then
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = { { Type = AI_ACTION_POSITION1, Position = pos } }
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.GotoEscort = function( BG, pWing, pPosition )
	if BG.Wings[pWing] then
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = { { Type = AI_ACTION_ESKORTE, Position = pos } }
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.AttackAndHyper = function( BG, pWing )
	if BG.Wings[pWing] then
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_CONTANGRIFF },
				{ Type = AI_ACTION_HYPERRAUM }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.MoveAndAttack = function( BG, pWing, pPosition )
	if BG.Wings[pWing] then
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_POSITION1, Position = pos },
				{ Type = AI_ACTION_CONTANGRIFF },
				{ Type = AI_ACTION_HYPERRAUM }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.AttackAndWait = function( BG, pWing )
	if BG.Wings[pWing] then
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_CONTANGRIFF },
				{ Type = AI_ACTION_WARTEN }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.MoveAndWait = function( BG, pWing, pPosition )
	if BG.Wings[pWing] then
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_POSITION1, Position = pos },
				{ Type = AI_ACTION_WARTEN }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.PlaceWing = function( BG, pWing, pPosition, pLookAt )
	if BG.Wings[pWing] then
		local front = NVector.Sub( { Vector1 = pLookAt, Vector2 = pPosition } ).Vector
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.Place( { Wing = BG.Wings[pWing], Position = pos, Front = front } )
	end
end


BattleLib.PlaceShip = function( BG, pShip, pPosition, pLookAt )
	if pShip then
		local front = NVector.Sub( { Vector1 = pLookAt, Vector2 = pPosition } ).Vector
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NShip.SetAlignment( { Ship = pShip, Position = pos, Front = front } )
	end
end


-- SpezialKamera fr AI-Schiff mit Spieler in Formation 21
BattleLib.DialogCamLeader = function( pCamObj, pAimObj, pFilter )
	local cam = NCamera.CreateSplineCam( {
		Name ="3dview/ActionCams/SplineKamera7.xml",
		Cam = pCamObj,
		Aim = pAimObj,
		CamOffset = { 25, 10, 20 },
		AimOffset = { 0, 0, 0 },
		Zoom = 0,
		Time = 2000,
		Immediate = true
	} ).Camera
	if pFilter then
		NCamera.AddFilter( { Camera = cam, Filter = pFilter } )
	end
	return cam
end


-- SpezialKamera fr AI-Schiff mit Spieler in Formation 21
BattleLib.DialogCamOther = function( pCamObj, pAimObj, pFilter )
	local cam = NCamera.CreateSplineCam( {
		Name ="3dview/ActionCams/SplineKamera7.xml",
		Cam = pCamObj,
		Aim = pAimObj,
		CamOffset = { -25, 15, -20 },
		AimOffset = { 0, 0, 0 },
		Zoom = 0,
		Time = 2000,
		Immediate = true
	} ).Camera
	if pFilter then
		NCamera.AddFilter( { Camera = cam, Filter = pFilter } )
	end
	return cam
end


BattleLib.AttackPlayer = function( BG, pWing, pPriority )
	if BG.Wings[pWing] then
		NWing.AddEnemy( {
			Wing = BG.Wings[pWing],
			Target = MissionLib.GetPlayerHandle(),
			Priority = pPriority
		} )
	end
end


BattleLib.AttackWing = function( BG, pWing, pOtherWing, pPriority )
	if BG.Wings[pWing] then
		if BG.Wings[pOtherWing] then
			NWing.AddEnemy( {
				Wing = BG.Wings[pWing],
				Target = BG.Wings[pOtherWing],
				Priority = pPriority
			} )
		elseif "Player" == pOtherWing then
			NWing.AddEnemy( {
				Wing = BG.Wings[pWing],
				Target = MissionLib.GetPlayerHandle(),
				Priority = pPriority
			} )
		end
	end
end


BattleLib.Retreat = function( BG, pWing, pPosition )
	if BG.Wings[pWing] then
		local pos = NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = { { Type = AI_ACTION_POSITION1, Position = pos } }
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.Land = function( BG, pWing, pHandle, pImmediate )
	if BG.Wings[pWing] and pHandle then
		local action = AI_ACTION_LANDEN
		if pImmediate then
			action = AI_ACTION_SOFORTLANDEN
		end
		if "TradeStation" == pHandle then
			pHandle = MissionLib.GetSystemTradeStation()
		end
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = { { Type = action , Handle = pHandle } }
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.GotoObjectAndAttack = function( BG, pWing, pHandle )
	if BG.Wings[pWing] then
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_OBJECT2, Handle = pHandle },
				{ Type = AI_ACTION_CONTANGRIFF },
				{ Type = AI_ACTION_WARTEN },
				{ Type = AI_ACTION_HYPERRAUM }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


-- BG Koordinaten in absolute Koordinaten umrechnen
BattleLib.CalcAbsPos = function( BG, pPosition )
	return NVector.Add( { Vector1 = BG.Center, Vector2 = pPosition } ).Vector
end


-- Den Spieler einen Wing anvisieren lassen
BattleLib.AutoTarget = function( BG, pWing, pMember )
	if BG.Wings[pWing] then
		local object
		if not pMember then
			object = MissionLib.GetWingLeader( BG.Wings[pWing] )
		else
			object = MissionLib.GetWingMember( BG.Wings[pWing], pMember )
		end
		if object then
			NPlayer.SelectTarget( { Object = object } )
		end
	end
end


BattleLib.GotoObjectAndWait = function( BG, pWing, pHandle )
	if BG.Wings[pWing] then
		NWing.AddAction( {
			Wing = BG.Wings[pWing],
			List = LIST_ACTIVE,
			First = true,
			Actions = {
				{ Type = AI_ACTION_OBJECT1, Handle = pHandle },
				{ Type = AI_ACTION_WARTEN }
			}
		} )
		NWing.ExecuteAi( { Wing = BG.Wings[pWing] } )
	end
end


BattleLib.LeaveGroup = function( BG, pWing )
	if pWing and BG.Wings[pWing] then
		if BG.Groups and BG.Defs[pWing] and BG.Defs[pWing].Group then
			NGroupAi.SubWings( {
				Group = BG.Groups[BG.Defs[pWing].Group],
				WingList = { BG.Wings[pWing] }
			} )
		end
	end
end


BattleLib.JoinGroup = function( BG, pWing, pGroup )
	if BG and BG.Groups[pGroup] and pWing and BG.Wings[pWing] then
		BattleLib.LeaveGroup( BG, pWing )
		NGroupAi.AddWings( {
			Group = BG.Groups[pGroup],
			WingList = { BG.Wings[pWing] }
		} )
	end
end


-- Zustzliche Funktionen einbinden
source "lua/mission/BattleLibEx.lua"

